/*******************************************************************************
* File Name: SCB.c
* Version 1.0
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCB_PVT.h"

#if(SCB_SCB_MODE_I2C_INC)
    #include "SCB_I2C_PVT.h"
#endif /* (SCB_SCB_MODE_I2C_INC) */

#if(SCB_SCB_MODE_SPI_INC || SCB_SCB_MODE_UART_INC)
    #include "SCB_SPI_UART_PVT.h"
#endif /* (SCB_SCB_MODE_SPI_INC || SCB_SCB_MODE_UART_INC) */


/**********************************
*    Run Time Configuration Vars
**********************************/

/* Stores internal component configuration for unconfigured mode */
#if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common config vars */
    volatile uint8 SCB_scbMode = SCB_SCB_MODE_UNCONFIG;
    volatile uint8 SCB_scbEnableWake;

    /* I2C config vars */
    volatile uint8 SCB_mode;
    volatile uint8 SCB_acceptAddr;

    /* SPI/UART config vars */
    volatile uint8 * SCB_rxBuffer;
    volatile uint8   SCB_rxDataBits;
    volatile uint32  SCB_rxBufferSize;

    volatile uint8 * SCB_txBuffer;
    volatile uint8   SCB_txDataBits;
    volatile uint32  SCB_txBufferSize;

#endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */


/**********************************
*     Common SCB Vars
**********************************/

uint8 SCB_initVar = 0u;
cyisraddress SCB_customIntrHandler = NULL;


/***************************************
*    Private Function Prototypes
***************************************/

static void SCB_ScbModeStop(void);


/*******************************************************************************
* Function Name: SCB_Init
********************************************************************************
*
* Summary:
*  Initializes SCB component to operate in one of selected configurations:
*  I2C, SPI, UART, EZI2C or EZSPI.
*  This function does not do any initialization when configuration is set to
*  Unconfigured SCB.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SCB_Init(void)
{
    #if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)
        if(SCB_SCB_MODE_UNCONFIG_RUNTM_CFG)
        {
            SCB_initVar = 0u; /* Clear init var */
        }
        else
        {
            /* Initialization was done before call */
        }

    #elif(SCB_SCB_MODE_I2C_CONST_CFG)
        SCB_I2CInit();

    #elif(SCB_SCB_MODE_SPI_CONST_CFG)
        SCB_SpiInit();

    #elif(SCB_SCB_MODE_UART_CONST_CFG)
        SCB_UartInit();

    #else

    #endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCB_Enable
********************************************************************************
*
* Summary:
*  Enables SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SCB_Enable(void)
{
    #if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)
        if(!SCB_SCB_MODE_UNCONFIG_RUNTM_CFG)
        {
            /* Enable SCB block, only if already configured */
            SCB_CTRL_REG |= SCB_CTRL_ENABLED;
        }
    #else
        SCB_CTRL_REG |= SCB_CTRL_ENABLED; /* Enable SCB block */
    #endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SCB_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call the component is enabled and ready for operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  SCB_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SCB_Start(void)
{
    if(0u == SCB_initVar)
    {
        SCB_initVar = 1u; /* Component was initialized */
        SCB_Init();       /* Initialize component      */
    }

    SCB_Enable();
}


/*******************************************************************************
* Function Name: SCB_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SCB_Stop(void)
{
    SCB_CTRL_REG &= (uint32) ~SCB_CTRL_ENABLED;  /* Disable SCB block */

    #if(SCB_SCB_IRQ_INTERNAL)
        SCB_SCB_IRQ_ClearPending(); /* Clear pending interrupt */
    #endif /* (SCB_SCB_IRQ_INTERNAL) */

    SCB_ScbModeStop(); /* Calls scbMode specific Stop function */
}


/*******************************************************************************
* Function Name: SCB_SetCustomInterruptHandler
********************************************************************************
*
* Summary:
*  Registers a function to be called by the internal interrupt handler.
*  First the function that is registered is called, then the internal interrupt
*  handler performs any operations such as software buffer management functions
*  before the interrupt returns.  It is user's responsibility to not break the
*  software buffer operations. Only one custom handler is supported, which is
*  the function provided by the most recent call.
*  At initialization time no custom handler is registered.
*
* Parameters:
*  func: Pointer to the function to register.
*        The value NULL indicates to remove the current custom interrupt
*        handler.
*
* Return:
*  None
*
*******************************************************************************/
void SCB_SetCustomInterruptHandler(cyisraddress func)
{
    SCB_customIntrHandler = func; /* Register interrupt handler */
}


/*******************************************************************************
* Function Name: SCB_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls Stop function for specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SCB_ScbModeStop(void)
{
    #if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)
        if(SCB_SCB_MODE_I2C_RUNTM_CFG)
        {
            SCB_I2CStop();
        }
    #elif(SCB_SCB_MODE_I2C_CONST_CFG)
        SCB_I2CStop();
    #else

    #endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */
}



#if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SCB_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets pins settings accordingly to selected operation mode.
    *  Only available in Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when specific mode of operation
    *  selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Submode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartTxRx:  Direction for UART.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SetPins(uint32 mode, uint32 subMode, uint32 uartTxRx)
    {
        uint32 hsiomSel[SCB_SCB_PINS_NUMBER];
        uint32 pinsDm  [SCB_SCB_PINS_NUMBER];
        uint32 i;

        /* Make all unused */
        for(i = 0u; i < SCB_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i] = SCB_HSIOM_DEF_SEL;
            pinsDm[i]   = SCB_PIN_DM_ALG_HIZ;
        }

        /* Choice the Dm and HSIOM */
        if(SCB_SCB_MODE_I2C == mode)
        {
            hsiomSel[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_HSIOM_I2C_SEL;
            hsiomSel[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_HSIOM_I2C_SEL;

            pinsDm[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_PIN_DM_OD_LO;
            pinsDm[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_PIN_DM_OD_LO;
        }
        else if(SCB_SCB_MODE_SPI == mode)
        {
            hsiomSel[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
            hsiomSel[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
            hsiomSel[SCB_SCLK_PIN_INDEX]        = SCB_HSIOM_SPI_SEL;

            if(SCB_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_PIN_DM_DIG_HIZ;
                pinsDm[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_PIN_DM_STRONG;
                pinsDm[SCB_SCLK_PIN_INDEX]        = SCB_PIN_DM_DIG_HIZ;

                #if(SCB_SS0_PIN)
                    /* Only SS0 is valid choice for Slave */
                    hsiomSel[SCB_SS0_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
                    pinsDm  [SCB_SS0_PIN_INDEX] = SCB_PIN_DM_DIG_HIZ;
                #endif /* (SCB_SS1_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_PIN_DM_STRONG;
                pinsDm[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_PIN_DM_DIG_HIZ;
                pinsDm[SCB_SCLK_PIN_INDEX]        = SCB_PIN_DM_STRONG;

                #if(SCB_SS0_PIN)
                    hsiomSel[SCB_SS0_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
                    pinsDm  [SCB_SS0_PIN_INDEX] = SCB_PIN_DM_STRONG;
                #endif /* (SCB_SS0_PIN) */

                #if(SCB_SS1_PIN)
                    hsiomSel[SCB_SS1_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
                    pinsDm  [SCB_SS1_PIN_INDEX] = SCB_PIN_DM_STRONG;
                #endif /* (SCB_SS1_PIN) */

                #if(SCB_SS2_PIN)
                    hsiomSel[SCB_SS2_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
                    pinsDm  [SCB_SS2_PIN_INDEX] = SCB_PIN_DM_STRONG;
                #endif /* (SCB_SS2_PIN) */

                #if(SCB_SS3_PIN)
                    hsiomSel[SCB_SS3_PIN_INDEX] = SCB_HSIOM_SPI_SEL;
                    pinsDm  [SCB_SS3_PIN_INDEX] = SCB_PIN_DM_STRONG;
                #endif /* (SCB_SS2_PIN) */
            }
        }
        else /* UART */
        {
            if(SCB_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_HSIOM_UART_SEL;
                pinsDm  [SCB_MISO_SDA_TX_PIN_INDEX] = SCB_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if(0u != (SCB_UART_RX & uartTxRx))
                {
                    hsiomSel[SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_HSIOM_UART_SEL;
                    pinsDm  [SCB_MOSI_SCL_RX_PIN_INDEX] = SCB_PIN_DM_DIG_HIZ;
                }

                if(0u != (SCB_UART_TX & uartTxRx))
                {
                    hsiomSel[SCB_MISO_SDA_TX_PIN_INDEX] = SCB_HSIOM_UART_SEL;
                    pinsDm  [SCB_MISO_SDA_TX_PIN_INDEX] = SCB_PIN_DM_STRONG;
                }
            }
        }

        /* Condfigure pins: set HSIOM and DM */
        /* Condfigure pins: DR registers configuration remains unchanged for cyfitter_cfg() */

        #if(SCB_MOSI_SCL_RX_PIN)
            SCB_SET_HSIOM_SEL(SCB_MOSI_SCL_RX_HSIOM_REG,
                                           SCB_MOSI_SCL_RX_HSIOM_MASK,
                                           SCB_MOSI_SCL_RX_HSIOM_POS,
                                           hsiomSel[SCB_MOSI_SCL_RX_PIN_INDEX]);
        #endif /* (SCB_MOSI_SCL_RX_PIN) */

        #if(SCB_MOSI_SCL_RX_WAKE_PIN)
            SCB_SET_HSIOM_SEL(SCB_MOSI_SCL_RX_WAKE_HSIOM_REG,
                                           SCB_MOSI_SCL_RX_WAKE_HSIOM_MASK,
                                           SCB_MOSI_SCL_RX_WAKE_HSIOM_POS,
                                           hsiomSel[SCB_MOSI_SCL_RX_WAKE_PIN_INDEX]);
        #endif /* (SCB_MOSI_SCL_RX_WAKE_PIN) */

        #if(SCB_MISO_SDA_TX_PIN)
            SCB_SET_HSIOM_SEL(SCB_MISO_SDA_TX_HSIOM_REG,
                                           SCB_MISO_SDA_TX_HSIOM_MASK,
                                           SCB_MISO_SDA_TX_HSIOM_POS,
                                           hsiomSel[SCB_MISO_SDA_TX_PIN_INDEX]);
        #endif /* (SCB_MOSI_SCL_RX_PIN) */

        #if(SCB_SCLK_PIN)
            SCB_SET_HSIOM_SEL(SCB_SCLK_HSIOM_REG, SCB_SCLK_HSIOM_MASK,
                                           SCB_SCLK_HSIOM_POS, hsiomSel[SCB_SCLK_PIN_INDEX]);
        #endif /* (SCB_SCLK_PIN) */

        #if(SCB_SS0_PIN)
            SCB_SET_HSIOM_SEL(SCB_SS0_HSIOM_REG, SCB_SS0_HSIOM_MASK,
                                           SCB_SS0_HSIOM_POS, hsiomSel[SCB_SS0_PIN_INDEX]);
        #endif /* (SCB_SS1_PIN) */

        #if(SCB_SS1_PIN)
            SCB_SET_HSIOM_SEL(SCB_SS1_HSIOM_REG, SCB_SS1_HSIOM_MASK,
                                           SCB_SS1_HSIOM_POS, hsiomSel[SCB_SS1_PIN_INDEX]);
        #endif /* (SCB_SS1_PIN) */

        #if(SCB_SS2_PIN)
            SCB_SET_HSIOM_SEL(SCB_SS2_HSIOM_REG, SCB_SS2_HSIOM_MASK,
                                           SCB_SS2_HSIOM_POS, hsiomSel[SCB_SS2_PIN_INDEX]);
        #endif /* (SCB_SS2_PIN) */

        #if(SCB_SS3_PIN)
            SCB_SET_HSIOM_SEL(SCB_SS3_HSIOM_REG,  SCB_SS3_HSIOM_MASK,
                                           SCB_SS3_HSIOM_POS, hsiomSel[SCB_SS3_PIN_INDEX]);
        #endif /* (SCB_SS3_PIN) */



        #if(SCB_MOSI_SCL_RX_PIN)
            SCB_spi_mosi_i2c_scl_uart_rx_SetDriveMode((uint8)
                                                                   pinsDm[SCB_MOSI_SCL_RX_PIN_INDEX]);
        #endif /* (SCB_MOSI_SCL_RX_PIN) */

        #if(SCB_MOSI_SCL_RX_WAKE_PIN)
        SCB_spi_mosi_i2c_scl_uart_rx_wake_SetDriveMode((uint8)
                                                                   pinsDm[SCB_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        /* Set interrupt on rising edge */
        SCB_SET_INCFG_TYPE(SCB_MOSI_SCL_RX_WAKE_INTCFG_REG,
                                        SCB_MOSI_SCL_RX_WAKE_INTCFG_TYPE_MASK,
                                        SCB_MOSI_SCL_RX_WAKE_INTCFG_TYPE_POS,
                                        SCB_INTCFG_TYPE_FALLING_EDGE);

        #endif /* (SCB_MOSI_SCL_RX_WAKE_PIN) */

        #if(SCB_MISO_SDA_TX_PIN)
            SCB_spi_miso_i2c_sda_uart_tx_SetDriveMode((uint8)
                                                                        pinsDm[SCB_MISO_SDA_TX_PIN_INDEX]);
        #endif /* (SCB_MOSI_SCL_RX_PIN) */

        #if(SCB_SCLK_PIN)
            SCB_spi_sclk_SetDriveMode((uint8) pinsDm[SCB_SCLK_PIN_INDEX]);
        #endif /* (SCB_SCLK_PIN) */

        #if(SCB_SS0_PIN)
            SCB_spi_ss0_SetDriveMode((uint8) pinsDm[SCB_SS0_PIN_INDEX]);
        #endif /* (SCB_SS0_PIN) */

        #if(SCB_SS1_PIN)
            SCB_spi_ss1_SetDriveMode((uint8) pinsDm[SCB_SS1_PIN_INDEX]);
        #endif /* (SCB_SS1_PIN) */

        #if(SCB_SS2_PIN)
            SCB_spi_ss2_SetDriveMode((uint8) pinsDm[SCB_SS2_PIN_INDEX]);
        #endif /* (SCB_SS2_PIN) */

        #if(SCB_SS3_PIN)
            SCB_spi_ss3_SetDriveMode((uint8) pinsDm[SCB_SS3_PIN_INDEX]);
        #endif /* (SCB_SS3_PIN) */
    }

#endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
