/*******************************************************************************
* File Name: SCB_SPI.c
* Version 1.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI mode.
*
* Note:
*
*******************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCB_PVT.h"
#include "SCB_SPI_UART_PVT.h"

#if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Config Structure Initialization
    ***************************************/

    const SCB_SPI_INIT_STRUCT SCB_configSpi =
    {
        SCB_SPI_MODE,
        SCB_SPI_SUB_MODE,
        SCB_SPI_CLOCK_MODE,
        SCB_SPI_OVS_FACTOR,
        SCB_SPI_MEDIAN_FILTER_ENABLE,
        SCB_SPI_LATE_MISO_SAMPLE_ENABLE,
        SCB_SPI_WAKE_ENABLE,
        SCB_SPI_RX_DATA_BITS_NUM,
        SCB_SPI_TX_DATA_BITS_NUM,
        SCB_SPI_BITS_ORDER,
        SCB_SPI_TRANSFER_SEPARATION,
        0u,
        NULL,
        0u,
        NULL,
        SCB_SPI_INTERRUPT_ENABLE,
        SCB_SPI_INTR_RX_MASK,
        SCB_SPI_RX_TRIGGER_LEVEL,
        SCB_SPI_INTR_TX_MASK,
        SCB_SPI_TX_TRIGGER_LEVEL
    };


    /*******************************************************************************
    * Function Name: SCB_SpiInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for SPI operation.
    *
    * Parameters:
    *  config:  Pointer to a structure that contains the following ordered list of
    *           fields. These fields match the selections available in the
    *           customizer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiInit(const SCB_SPI_INIT_STRUCT *config)
    {
        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due bad fucntion parameter */
        }
        else
        {
            /* Configure pins */
            SCB_SetPins(SCB_SCB_MODE_SPI, config->mode, SCB_DUMMY_PARAM);

            /* Store internal configuration */
            SCB_scbMode       = (uint8) SCB_SCB_MODE_SPI;
            SCB_scbEnableWake = (uint8) config->enableWake;

            /* Set RX direction internal variables */
            SCB_rxBuffer      =         config->rxBuffer;
            SCB_rxDataBits    = (uint8) config->rxDataBits;
            SCB_rxBufferSize  = (uint8) config->rxBufferSize;

            /* Set TX direction internal variables */
            SCB_txBuffer      =         config->txBuffer;
            SCB_txDataBits    = (uint8) config->txDataBits;
            SCB_txBufferSize  = (uint8) config->txBufferSize;


            /* Configure SPI interface */
            SCB_CTRL_REG     = SCB_GET_CTRL_OVS(config->oversample)        |
                                            SCB_GET_CTRL_EC_AM_MODE(config->enableWake) |
                                            SCB_CTRL_SPI;

            SCB_SPI_CTRL_REG = SCB_GET_SPI_CTRL_CONTINUOUS    (config->transferSeperation)  |
                                            SCB_GET_SPI_CTRL_SELECT_PRECEDE(config->submode & 
                                                                          SCB_SPI_MODE_TI_PRECEDES_MASK) |
                                            SCB_GET_SPI_CTRL_SCLK_MODE     (config->sclkMode)            |
                                            SCB_GET_SPI_CTRL_LATE_MISO_SAMPLE(config->enableLateSampling)|
                                            SCB_GET_SPI_CTRL_SUB_MODE      (config->submode)             |
                                            SCB_GET_SPI_CTRL_MASTER_MODE   (config->mode);

            /* Configure RX direction */
            SCB_RX_CTRL_REG     =  SCB_GET_RX_CTRL_DATA_WIDTH(config->rxDataBits)         |
                                                SCB_GET_RX_CTRL_BIT_ORDER (config->bitOrder)           |
                                                SCB_GET_RX_CTRL_MEDIAN    (config->enableMedianFilter) |
                                                SCB_SPI_RX_CTRL;

            SCB_RX_FIFO_CTRL_REG = SCB_GET_RX_FIFO_CTRL_TRIGGER_LEVEL(config->rxTriggerLevel);

            /* Configure TX direction */
            SCB_TX_CTRL_REG      = SCB_GET_TX_CTRL_DATA_WIDTH(config->txDataBits) |
                                                SCB_GET_TX_CTRL_BIT_ORDER (config->bitOrder)   |
                                                SCB_SPI_TX_CTRL;

            SCB_TX_FIFO_CTRL_REG = SCB_GET_TX_FIFO_CTRL_TRIGGER_LEVEL(config->txTriggerLevel);

            /* Configure interrupt sources */
            SCB_INTR_I2C_EC_MASK_REG = SCB_NO_INTR_SOURCES;
            SCB_INTR_SPI_EC_MASK_REG = SCB_NO_INTR_SOURCES;
            SCB_INTR_SLAVE_MASK_REG  = SCB_GET_SPI_INTR_SLAVE_MASK(config->rxInterruptMask);
            SCB_INTR_MASTER_MASK_REG = SCB_GET_SPI_INTR_MASTER_MASK(config->txInterruptMask);
            SCB_INTR_RX_MASK_REG     = SCB_GET_SPI_INTR_RX_MASK(config->rxInterruptMask);
            SCB_INTR_TX_MASK_REG     = SCB_GET_SPI_INTR_TX_MASK(config->txInterruptMask);

            /* Configure interrupt source */
            SCB_SCB_IRQ_StartEx(&SCB_SPI_UART_ISR);
            if(0u == config->enableInterrupt)
            {
                SCB_SCB_IRQ_Disable();
            }


            /* Clear RX buffer indexes */
            SCB_rxBufferHead     = 0u;
            SCB_rxBufferTail     = 0u;
            SCB_rxBufferOverflow = 0u;

            /* Clrea TX buffer indexes */
            SCB_txBufferHead = 0u;
            SCB_txBufferTail = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SCB_SpiInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for SPI operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiInit(void)
    {
        /* Configure SPI interface */
        SCB_CTRL_REG     = SCB_SPI_DEFAULT_CTRL;
        SCB_SPI_CTRL_REG = SCB_SPI_DEFAULT_SPI_CTRL;

        /* Configure TX and RX direction */
        SCB_RX_CTRL_REG      = SCB_SPI_DEFAULT_RX_CTRL;
        SCB_RX_FIFO_CTRL_REG = SCB_SPI_DEFAULT_RX_FIFO_CTRL;

        /* Configure TX and RX direction */
        SCB_TX_CTRL_REG      = SCB_SPI_DEFAULT_TX_CTRL;
        SCB_TX_FIFO_CTRL_REG = SCB_SPI_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt sources */
        SCB_INTR_I2C_EC_MASK_REG = SCB_SPI_DEFAULT_INTR_I2C_EC_MASK;
        SCB_INTR_SPI_EC_MASK_REG = SCB_SPI_DEFAULT_INTR_SPI_EC_MASK;
        SCB_INTR_SLAVE_MASK_REG  = SCB_SPI_DEFAULT_INTR_SLAVE_MASK;
        SCB_INTR_MASTER_MASK_REG = SCB_SPI_DEFAULT_INTR_MASTER_MASK;
        SCB_INTR_RX_MASK_REG     = SCB_SPI_DEFAULT_INTR_RX_MASK;
        SCB_INTR_TX_MASK_REG     = SCB_SPI_DEFAULT_INTR_TX_MASK;


        /* Configure interrupt source */
        #if(SCB_SCB_IRQ_INTERNAL)
            SCB_SCB_IRQ_StartEx(&SCB_SPI_UART_ISR);

            #if(SCB_SPI_INTERRUPT_DISABLE)
                SCB_SCB_IRQ_Disable();
            #endif /* (SCB_SPI_INTERRUPT_DISABLE) */
        #endif /* (SCB_SCB_IRQ_INTERNAL) */


        #if(SCB_INTERNAL_RX_SW_BUFFER_CONST)
            SCB_rxBufferHead     = 0u;
            SCB_rxBufferTail     = 0u;
            SCB_rxBufferOverflow = 0u;
        #endif /* (SCB_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCB_INTERNAL_TX_SW_BUFFER_CONST)
            SCB_txBufferHead = 0u;
            SCB_txBufferTail = 0u;
        #endif /* (SCB_INTERNAL_TX_SW_BUFFER_CONST) */
    }

#endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SCB_SetActiveSlaveSelect
********************************************************************************
*
* Summary:
*  Selects one of the four SPI slave select signals.
*  The component should be in one of the following states to change the active
*  slave select signal source correctly:
*   - the component is disabled
*   - the component has completed all transactions (TX FIFO is empty and the
*     SpiDone flag is set)
*  This function does not check that these conditions are met.
*  At initialization time the active slave select line is slave select 0.
*
* Parameters:
*  activeSelect: The four lines available to utilize Slave Select function.
*
* Return:
*  None
*
*******************************************************************************/
void SCB_SpiSetActiveSlaveSelect(uint32 activeSelect)
{
    uint32 spiCtrl;
    
    spiCtrl = SCB_SPI_CTRL_REG;
    
    spiCtrl &= (uint32) ~SCB_SPI_CTRL_SLAVE_SELECT_MASK; /* Clear SS bits */
    spiCtrl |= (uint32)  SCB_GET_SPI_CTRL_SS(activeSelect);
    
    SCB_SPI_CTRL_REG = spiCtrl;
}


/*******************************************************************************
* Function Name: SCB_SpiSaveConfig
********************************************************************************
*
* Summary:
*  Wakeup disabled: does nothing.
*  Wakeup  enabled: clears SCB_backup.enableState to keep component enabled
*  while DeepSleep. The SCB_INTR_SPI_EC_WAKE_UP enabled while initialization
*  and tracks in active mode therefore it does not require to be cleared.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  None
*
*******************************************************************************/
void SCB_SpiSaveConfig(void)
{
    #if(SCB_CHECK_SPI_WAKE_ENABLE)
    {
        SCB_backup.enableState = 0u; /* Keep SCB enabled */
        
        /* Clear wake-up before enable */
        SCB_ClearSpiExtClkInterruptSource(SCB_INTR_SPI_EC_WAKE_UP);
        
        /* Enable interrupt to wakeup the device */
        SCB_SetSpiExtClkInterruptMode(SCB_INTR_SPI_EC_WAKE_UP);
    }
    #endif
}


/*******************************************************************************
* Function Name: SCB_SpiRestoreConfig
********************************************************************************
*
* Summary:
*  Does nothing.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  None
*
*******************************************************************************/
void SCB_SpiRestoreConfig(void)
{
    #if(SCB_CHECK_SPI_WAKE_ENABLE)
    {
        /* Disable interrupt to wakeup the device */
        SCB_SetSpiExtClkInterruptMode(SCB_NO_INTR_SOURCES);
    }
    #endif
}


/* [] END OF FILE */
