/*******************************************************************************
* File Name: SCB_SPI_UART.c
* Version 1.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCB_SPI_UART_PVT.h"


/***************************************
*        SPI/UART Private Vars
***************************************/

#if(SCB_INTERNAL_RX_SW_BUFFER_CONST)
    volatile uint32 SCB_rxBufferHead;
    volatile uint32 SCB_rxBufferTail;
    volatile uint8  SCB_rxBufferOverflow;
#endif /* (SCB_INTERNAL_RX_SW_BUFFER_CONST) */

#if(SCB_INTERNAL_TX_SW_BUFFER_CONST)
    volatile uint32 SCB_txBufferHead;
    volatile uint32 SCB_txBufferTail;
#endif /* (SCB_INTERNAL_TX_SW_BUFFER_CONST) */

#if(SCB_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 SCB_rxBufferInternal[SCB_RX_BUFFER_SIZE];
#endif /* (SCB_INTERNAL_RX_SW_BUFFER) */

#if(SCB_INTERNAL_TX_SW_BUFFER)
    volatile uint8 SCB_txBufferInternal[SCB_TX_BUFFER_SIZE];
#endif /* (SCB_INTERNAL_TX_SW_BUFFER) */


#if(SCB_RX_DIRECTION)

    /*******************************************************************************
    * Function Name: SCB_SpiUartReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Retrieves the next data element from the receive buffer. Undefined data will
    *  be returned if the RX buffer is empty.
    *  Call SCB_SpiUartGetRxBufferSize() to return buffer size.
    *   - RX software buffer disabled: Returns data element retrieved from RX FIFO.
    *   - RX software buffer enabled: Returns data element from the software
    *     receive buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Next data element from the receive buffer.
    *
    * Global Variables:
    *  Look into SCB_SpiInit for description.
    *
    *******************************************************************************/
    uint32 SCB_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;
        
        #if(SCB_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (SCB_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_RX_SW_BUFFER)
        {
            if(SCB_rxBufferHead != SCB_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (SCB_rxBufferTail + 1u);

                if(SCB_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data fron RX software buffer */
                rxData = SCB_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                SCB_rxBufferTail = locTail;
            }
        }
        #else
        {
            rxData = SCB_RX_FIFO_RD_REG; /* Read data from RX FIFO */
        }
        #endif

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: SCB_SpiUartGetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in receive buffer.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of received data elements
    *
    *******************************************************************************/
    uint32 SCB_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
        #if(SCB_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locHead;
        #endif /* (SCB_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_RX_SW_BUFFER)
        {
            locHead = SCB_rxBufferHead;

            if(locHead >= SCB_rxBufferTail)
            {
                size = (locHead - SCB_rxBufferTail);
            }
            else
            {
                size = (locHead + (SCB_RX_BUFFER_SIZE - SCB_rxBufferTail));
            }
        }
        #else
        {
            size = SCB_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: SCB_SpiUartClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clear the receive buffer and RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiUartClearRxBuffer(void)
    {
        #if(SCB_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (SCB_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_RX_SW_BUFFER)
        {
            intSourceMask = SCB_SpiUartDisableIntRx();

            SCB_CLEAR_RX_FIFO;

            /* Flush RX software buffer */
            SCB_rxBufferHead     = SCB_rxBufferTail;
            SCB_rxBufferOverflow = 0u;

            /* End RX transfer */
            SCB_ClearRxInterruptSource(SCB_INTR_RX_ALL);

            SCB_SpiUartEnableIntRx(intSourceMask);
        }
        #else
        {
            SCB_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (SCB_RX_DIRECTION) */


#if(SCB_TX_DIRECTION)

    /*******************************************************************************
    * Function Name: SCB_SpiUartWriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    * Parameters:
    *  txDataByte: the data to be transmitted.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiUartWriteTxData(uint32 txDataByte)
    {
        #if(SCB_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locHead;
            uint32 intSourceMask;
        #endif /* (SCB_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_TX_SW_BUFFER)
        {
            /* Head index to put data */
            locHead = (SCB_txBufferHead + 1u);

            /* Adjust TX software buffer index */
            if(SCB_TX_BUFFER_SIZE == locHead)
            {
                locHead = 0u;
            }

            while(locHead == SCB_txBufferTail)
            {
                /* Wait for space in the TX software buffer */
            }

            /* The TX software buffer has at least one room */

            if((SCB_txBufferHead == SCB_txBufferTail) &&
               (SCB_FIFO_SIZE != SCB_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                SCB_TX_FIFO_WR_REG = txDataByte;
            }
            /* Put data in the TX software buffer */
            else
            {
                /* Clear old status of INTR_TX_EMPTY. It sets at the end of transfer: TX FIFO empty. */
                SCB_ClearTxInterruptSource(SCB_INTR_TX_EMPTY);

                SCB_PutWordInTxBuffer(locHead, txDataByte);

                SCB_txBufferHead = locHead;

                /* Enable interrupt to transmit */
                intSourceMask  = SCB_INTR_TX_EMPTY;
                intSourceMask |= SCB_GetTxInterruptMode();
                SCB_SpiUartEnableIntTx(intSourceMask);
            }
        }
        #else
        {
            while(SCB_FIFO_SIZE == SCB_GET_TX_FIFO_ENTRIES)
            {
                /* Block while TX FIFO is FULL */
            }

            SCB_TX_FIFO_WR_REG = txDataByte;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: SCB_SpiUartPutArray
    ********************************************************************************
    *
    * Summary:
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * Parameters:
    *  wrBuf:  pointer to an array with data to be placed in transmit buffer.
    *  count:  number of data elements to be placed in the transmit buffer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for(i=0u; i < count; i++)
        {
            SCB_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: SCB_SpiUartGetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of elements currently in the transmit buffer.
    *  TX software buffer disabled: returns the number of used entries in TX FIFO.
    *  TX software buffer enabled: returns the number of elements currently used
    *  in the transmit buffer. This number does not include used entries in the
    *  TX FIFO. The transmit buffer size is zero until the TX FIFO is full.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of data elements ready to transmit.
    *
    *******************************************************************************/
    uint32 SCB_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
        #if(SCB_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (SCB_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = SCB_txBufferTail;

            if(SCB_txBufferHead >= locTail)
            {
                size = (SCB_txBufferHead - locTail);
            }
            else
            {
                size = (SCB_txBufferHead + (SCB_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = SCB_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: SCB_SpiUartClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the transmit buffer and TX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_SpiUartClearTxBuffer(void)
    {
        #if(SCB_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (SCB_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(SCB_CHECK_TX_SW_BUFFER)
        {
            intSourceMask = SCB_SpiUartDisableIntTx();

            SCB_CLEAR_TX_FIFO;

            /* Flush TX software buffer */
            SCB_txBufferHead = SCB_txBufferTail;

            /* End TX transfer if it is in progress */
            intSourceMask &= (uint32) ~SCB_INTR_TX_EMPTY;

            SCB_SpiUartEnableIntTx(intSourceMask);
        }
        #else
        {
            SCB_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (SCB_TX_DIRECTION) */


/*******************************************************************************
* Function Name: SCB_SpiUartDisableIntRx
********************************************************************************
*
* Summary:
*  Disables RX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns RX interrupt soureces enabled before function call.
*
*******************************************************************************/
uint32 SCB_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = SCB_GetRxInterruptMode();

    SCB_SetRxInterruptMode(SCB_NO_INTR_SOURCES);

    return(intSource);
}


/*******************************************************************************
* Function Name: SCB_SpiUartDisableIntTx
********************************************************************************
*
* Summary:
*  Disables TX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns TX interrupt soureces enabled before function call.
*
*******************************************************************************/
uint32 SCB_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = SCB_GetTxInterruptMode();

    SCB_SetTxInterruptMode(SCB_NO_INTR_SOURCES);

    return(intSourceMask);
}


#if(SCB_SCB_MODE_UNCONFIG_CONST_CFG)

    /*******************************************************************************
    * Function Name: SCB_PutWordInRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores byte/word into the RX buffer.
    *  Only available in Unconfigured operation mode.
    *
    * Parameters:
    *  index:      index to store data byte/word in the RX buffer.
    *  rxDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in the buffer */
        if(SCB_ONE_BYTE_WIDTH == SCB_rxDataBits)
        {
            SCB_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            SCB_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            SCB_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCB_GetWordFromRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from RX buffer.
    *  Only available in Unconfigured operation mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 SCB_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if(SCB_ONE_BYTE_WIDTH == SCB_rxDataBits)
        {
            value = SCB_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) SCB_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)SCB_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }


    /*******************************************************************************
    * Function Name: SCB_PutWordInTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores byte/word into the TX buffer.
    * Only available in Unconfigured operation mode.
    *
    * Parameters:
    *  idx:        index to store data byte/word in the TX buffer.
    *  txDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in the buffer */
        if(SCB_ONE_BYTE_WIDTH == SCB_txDataBits)
        {
            SCB_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            SCB_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            SCB_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SCB_GetWordFromTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from TX buffer.
    *  Only available in Unconfigured operation mode.
    *
    * Parameters:
    *  idx: index to get data byte/word from the TX buffer.
    *
    * Return:
    *  Returns byte/word read from TX buffer.
    *
    *******************************************************************************/
    uint32 SCB_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if(SCB_ONE_BYTE_WIDTH == SCB_txDataBits)
        {
            value = (uint32) SCB_txBuffer[idx];
        }
        else
        {
            value  = (uint32) SCB_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) SCB_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }

#endif /* (SCB_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
